@@ -2,7 +2,9 @@ from typing import List
 from shapely.geometry import LineString
 from pyproj import Transformer
 import numpy as np
+import numba as nb
 import matplotlib.pyplot as plt
+from pygeoshape.utils import fast_intersection_append
 
 
 class GeoLineString:
@@ -45,18 +47,18 @@ class GeoLineString:
 
         self.np_coords = np.array(self.xy_coords)
         self.length = self.geolinestring_length()
-        dimensions = len(self.xy_coords[0])
+        self.dimensions = len(self.xy_coords[0])
 
         # need linestring for each dimension
         # x, y, [z]
         # x, z, [y]
         # y, z, [x]
 
-        if dimensions == 2:
+        if self.dimensions == 2:
             self.xy = LineString([(coord[0], coord[1])
                                  for coord in self.xy_coords])
 
-        if dimensions == 3:
+        if self.dimensions == 3:
 
             self.xy = LineString(
                 [(coord[0], coord[1], coord[2]) for coord in self.xy_coords]
@@ -144,7 +146,7 @@ class GeoLineString:
 
         Args:
         ----
-        geo_obj (GeoLineString): Second GeoLineString for comparision
+        geo_obj (GeoLineString): Second GeoLineString for comparison
 
         Returns:
         -------
@@ -164,7 +166,7 @@ class GeoLineString:
 
         Args:
         ----
-        geo_obj (GeoLineString): Second GeoLineString for comparision
+        geo_obj (GeoLineString): Second GeoLineString for comparison
         lonlat (bool): Format of output coordinates. True returns the coordinates in longitude, latitude. False returns coordinates in x, y
 
         Returns:
@@ -172,69 +174,55 @@ class GeoLineString:
         intersection (List): Intersection coordinates
 
         """
-        xy_check = self.xy.intersects(geo_obj.xy)
-        xz_check = self.xz.intersects(geo_obj.xz)
-        yz_check = self.yz.intersects(geo_obj.yz)
 
-        intersection = []
+        intersection = nb.typed.List.empty_list(
+            nb.types.UniTuple(nb.float64, self.dimensions))
 
-        if all([xy_check, xz_check, yz_check]):
+        inter1 = self.xy.intersection(geo_obj.xy)
+        inter2 = self.xz.intersection(geo_obj.xz)
+        inter3 = self.yz.intersection(geo_obj.yz)
 
-            inter1 = self.xy.intersection(geo_obj.xy)
-            inter2 = self.xz.intersection(geo_obj.xz)
-            inter3 = self.yz.intersection(geo_obj.yz)
+        if not hasattr(inter1, "geoms"):
+            xy = np.array(inter1.coords)[0]
+            n_xy_intersections = 1
 
-            if not hasattr(inter1, "geoms"):
-                xy = list(inter1.coords)[0]
-                n_xy_intersections = 1
-            else:
-                n_xy_intersections = len(inter1.geoms)
+        else:
+            n_xy_intersections = len(inter1.geoms)
 
-            if not hasattr(inter2, "geoms"):
-                xz = list(inter2.coords)[0]
-                n_xz_intersections = 1
-            else:
-                n_xz_intersections = len(inter2.geoms)
+        if not hasattr(inter2, "geoms"):
+            xz = np.array(inter2.coords)[0]
+            n_xz_intersections = 1
+        else:
+            n_xz_intersections = len(inter2.geoms)
 
-            if not hasattr(inter3, "geoms"):
-                yz = list(inter3.coords)[0]
-                n_yz_intersections = 1
-            else:
-                n_yz_intersections = len(inter3.geoms)
+        if not hasattr(inter3, "geoms"):
+            yz = np.array(inter3.coords)[0]
+            n_yz_intersections = 1
+        else:
+            n_yz_intersections = len(inter3.geoms)
 
-            for i in range(n_xy_intersections):
+        for i in range(n_xy_intersections):
 
-                for j in range(n_xz_intersections):
+            for j in range(n_xz_intersections):
 
-                    for k in range(n_yz_intersections):
+                for k in range(n_yz_intersections):
 
-                        if hasattr(inter1, "geoms"):
-                            xy = list(inter1.geoms[i].coords)
-                        else:
-                            xy = list(inter1.coords)
+                    if hasattr(inter1, "geoms"):
+                        xy = np.array(inter1.geoms[i].coords)
+                    else:
+                        xy = np.array(inter1.coords)
 
-                        if hasattr(inter2, "geoms"):
-                            xz = list(inter2.geoms[j].coords)
-                        else:
-                            xz = list(inter2.coords)
+                    if hasattr(inter2, "geoms"):
+                        xz = np.array(inter2.geoms[j].coords)
+                    else:
+                        xz = np.array(inter2.coords)
 
-                        if hasattr(inter3, "geoms"):
-                            yz = list(inter3.geoms[k].coords)
-                        else:
-                            yz = list(inter3.coords)
+                    if hasattr(inter3, "geoms"):
+                        yz = np.array(inter3.geoms[k].coords)
+                    else:
+                        yz = np.array(inter3.coords)
 
-                        for ii in range(len(xy)):
-
-                            x_y_z = xy[ii]
-                            x_z_y = (x_y_z[0], x_y_z[2], x_y_z[1])
-
-                            if x_z_y in xz:
-
-                                y_z_x = (x_y_z[1], x_y_z[2], x_y_z[0])
-
-                                if y_z_x in yz:
-
-                                    intersection.append(x_y_z)
+                    fast_intersection_append(xy, xz, yz, intersection)
 
             intersection = np.unique(intersection, axis=0)
 
